import {
  reactive,
  computed,
  unref,
  isRef,
  watch,
  getCurrentInstance,
  inject,
  provide,
} from 'vue'
import minifier from 'minify-groq'

const cache = reactive({})
function ensureInstance() {
  const instance = getCurrentInstance()
  if (!instance) throw new Error('You must call this from within a component')
  return instance
}
function useCache(key, fetcher, options = {}) {
  const { initialValue = null, deduplicate = false } = options
  function initialiseCache({
    key: key2,
    value,
    error: error2 = null,
    status: status2 = 'initialised',
    time = new Date().getTime(),
  }) {
    cache[key2] = [value, status2, time, error2, null]
  }
  function verifyKey(key2) {
    const emptyCache = !(key2 in cache)
    if (emptyCache) initialiseCache({ key: key2, value: initialValue })
    return emptyCache || cache[key2][1] === 'initialised'
  }
  function setCache({
    key: key2,
    value = (_a => ((_a = cache[key2]) == null ? void 0 : _a[0]))() ||
      initialValue,
    status: status2 = (_b => ((_b = cache[key2]) == null ? void 0 : _b[1]))(),
    error: error2 = null,
    promise = (_c => ((_c = cache[key2]) == null ? void 0 : _c[4]))() || null,
  }) {
    if (!(key2 in cache)) initialiseCache({ key: key2, value, status: status2 })
    cache[key2][0] = value
    cache[key2][1] = status2
    cache[key2][2] = new Date().getTime()
    cache[key2][3] = error2
    cache[key2][4] = promise
  }
  function fetch(query = unref(key), force) {
    if (
      !force &&
      query &&
      cache[query] &&
      cache[query][1] !== 'error' &&
      (cache[query][0] !== initialValue ||
        cache[query][4] instanceof Promise) &&
      deduplicate &&
      (deduplicate === true ||
        deduplicate < new Date().getTime() - cache[query][2])
    )
      return cache[query][4] instanceof Promise
        ? cache[query][4]
        : Promise.resolve(cache[query][0])
    const promise = fetcher(query)
    setCache({ key: query, status: 'loading', promise })
    promise
      .then(value =>
        setCache({
          key: query,
          value,
          status: 'client loaded',
          promise: null,
        })
      )
      .catch(error2 => setCache({ key: query, status: 'error', error: error2 }))
    return promise
  }
  const data = computed(() => {
    const k = unref(key)
    if (!k) return initialValue
    verifyKey(k)
    return cache[k] && cache[k][0]
  })
  const status = computed(() => {
    const k = unref(key)
    if (!k) return 'initialised'
    verifyKey(k)
    return cache[k][1]
  })
  const error = computed(() => {
    const k = unref(key)
    if (!k) return null
    verifyKey(k)
    return cache[k][3]
  })
  if (isRef(key)) {
    watch(
      key,
      key2 => {
        fetch(key2, verifyKey(key2))
      },
      { immediate: true }
    )
  } else {
    fetch(key, verifyKey(key))
  }
  return {
    setCache,
    triggerFetch: fetch,
    fetch,
    data,
    status,
    error,
  }
}

const imageBuilderSymbol = Symbol('Sanity image URL builder')
function useSanityImage(image, options, widths = [300, 600, 1200, 1920]) {
  const builder2 = inject(imageBuilderSymbol)
  if (!builder2)
    throw new Error(
      'You must call useSanityClient before using sanity resources in this project.'
    )
  function getImageUrl(image2, width, { quality = 82, fit = 'min' }) {
    return builder2
      .image(image2.url)
      .width(Math.round(width))
      .quality(quality)
      .fit(fit)
      .url()
  }
  const result = computed(() => ({
    src: image.value.url,
    ...(image.value.dimensions
      ? {
          srcset: [
            ...widths.map(
              width =>
                `${getImageUrl(image.value, width, options || {})} ${width}w`
            ),
            `${image.value.url} ${image.value.dimensions.width}w`,
          ].join(', '),
          placeholder: '',
        }
      : {
          srcset: [
            ...widths.map(
              width =>
                `${getImageUrl(image.value, width, options || {})} ${width}w`
            ),
          ].join(', '),
          placeholder: '',
        }),
  }))
  return result
}

const clientSymbol = Symbol('Sanity client')
const previewClientSymbol = Symbol('Sanity client for previews')
const optionsSymbol = Symbol('Default query options')
function useSanityFetcher(
  query,
  initialValue = null,
  mapper = result => result,
  queryOptions
) {
  const client = inject(clientSymbol)
  const defaultOptions = inject(optionsSymbol, {})
  const options = {
    ...defaultOptions,
    initialValue,
    ...queryOptions,
  }
  if (!client)
    throw new Error(
      'You must call useSanityClient before using sanity resources in this project.'
    )
  const computedQuery =
    typeof query === 'string'
      ? minifier(query).replace(/\n/g, ' ')
      : computed(() => minifier(query() || '').replace(/\n/g, ' '))
  const { data, status, setCache, error, fetch } = useCache(
    computedQuery,
    query2 =>
      query2
        ? client.fetch(query2).then(mapper)
        : Promise.resolve(initialValue),
    options
  )
  if (options.listen) {
    const previewClient = inject(previewClientSymbol, client)
    if ('listen' in previewClient) {
      const listenOptions =
        typeof options.listen === 'boolean' ? void 0 : options.listen
      const subscribe = query2 =>
        previewClient.listen(query2, listenOptions).subscribe(
          event =>
            event.result &&
            setCache({
              key: query2,
              value: event.result,
            })
        )
      if (isRef(computedQuery)) {
        watch(
          computedQuery,
          query2 => {
            const subscription = subscribe(query2)
            const unwatch = watch(
              computedQuery,
              newQuery => {
                if (newQuery !== query2) {
                  subscription.unsubscribe()
                  unwatch()
                }
              },
              { immediate: true }
            )
          },
          { immediate: true }
        )
      } else {
        subscribe(computedQuery)
      }
    }
  }
  return { data, status, error, fetch }
}
function useSanityQuery(
  builder,
  initialValue = null,
  mapper = result => result,
  options
) {
  const query =
    'use' in builder ? () => builder.use()[0] : () => builder().use()[0]
  const type = 'use' in builder ? builder.use()[1] : builder().use()[1]
  return useSanityFetcher(query, initialValue || type, mapper, options)
}

function useSanityClient(config, supportPreview = false, defaultOptions = {}) {
  ensureInstance()
  const sanityClient = require('@sanity/client')
  const imageUrlBuilder = require('@sanity/image-url')
  const client2 = sanityClient(config)
  const imageBuilder = imageUrlBuilder(config)
  provide(clientSymbol, client2)
  provide(imageBuilderSymbol, imageBuilder)
  provide(optionsSymbol, defaultOptions)
  if (supportPreview) {
    const previewClient = sanityClient({
      ...config,
      useCdn: false,
      token: void 0,
      withCredentials: true,
    })
    provide(previewClientSymbol, previewClient)
  }
}
function useCustomClient(client2, defaultOptions = {}) {
  ensureInstance()
  provide(clientSymbol, client2)
  provide(optionsSymbol, defaultOptions)
}
function fetch(query2) {
  ensureInstance()
  const client2 = inject(clientSymbol)
  if (!client2)
    throw new Error(
      'You must call useSanityClient before using sanity resources in this project.'
    )
  return client2.fetch(query2)
}

export {
  fetch,
  useCache,
  useCustomClient,
  useSanityClient,
  useSanityFetcher,
  useSanityImage,
  useSanityQuery,
}
